   '***********************************************************************
   ' MikroPAJkA 2011
   ' v 1.0-PWM (z 10.9.2011)
   '
   ' - plynula regulace vyhrivani telesa a udrzovani nastavene teploty (merenim kazdou 0,5s)
   ' - stand-by rezim s udrzovaci teplotou 120C a vypnutim podsviceni
   ' - moznost vypinat piezo
   ' - rychle nastaveni teploty - skok po 50C
   ' - zvukova signalizace dosazeni teploty pri prvnim zahrati
   ' - hlidani odlozene rucky ve stojanku, po 10,12, a 14 min zvukove upozorneni, po 15minute vypne topeni, podsviceni a lze probudit jen stiskem tlacitka
   '
   ' Autor : Pavel Jank
   ' www.paja-trb.cz
   ' paja@paja-trb.cz
   '
   '***********************************************************************

'$sim                                                        ' Odstranit pi programovn - zrychlen simulace
$regfile = "M8def.dat"                                      ' ATmega8
$crystal = 12000000                                         ' 12MHz - crystal oscillator value

Config Lcd = 16 * 2                                         'Typ LCD
Config Lcdpin = Pin , Db4 = Portc.2 , Db5 = Portc.3 , Db6 = Portc.1 , Db7 = Portc.0 , E = Portc.5 , Rs = Portc.4       'Konfigurace pinu LCD
Cursor Off                                                  'vypni kurzor

Config Timer1 = Timer , Prescale = 256                      '1krok tae = 1 / 12 (MHz) * 255 (peddli) = 21,25 mikrosec
Timer1 = 42005                                              'Poten hodnota registru TCNT1 (t od 0 do max. 65535 "16bit" -> 21,25mikrosec * 23529 = 0,5s)
Enable Timer1                                               'povol timer1
Stop Timer1                                                 'zastav timer1

Config Timer2 = Pwm , Prescale = 128 , Compare Pwm = Clear Up , Pwm = On       'Timer2 jako PWM, dlika = 256, pi dosaen shody resetuj, PWM zapnuto
Enable Timer2                                               'povol timer2
Stop Timer2                                                 'zastav timer2
Enable Oc2                                                  'povol kompartor hodnoty registru TCNT2
Topeni Alias Ocr2                                           ' = Hodnota Pwm
Config Portb.3 = Output                                     'Vstup PWM - vyhrivani telesa

Stojanek Alias Pinb.4                                       'hlidani doby odlozene rucky pajky a zbytecneho zahrivani (10,12,14min piezo, po 15.minute vypne topeni)
Config Portb.4 = Input                                      'soust ISP konektoru MISO
Stojanek = 1                                                'pull-up

'MAX6675             MISO = PB.0         SCK = PB.2         /CS = PB.1
Config Pinb.0 = Input
Config Pinb.1 = Output                                      'PortB.1 jako vstup pro CS MAX6675
Config Pinb.2 = Output
Config Spi = Soft , Din = Pinb.0 , Dout = Portb.1 , Ss = None , Clock = Portb.2       'Kobfigurace SPI, softwarov
Max_cs Alias Portb.1 : Set Max_cs
Max_sck Alias Portb.2 : Reset Max_sck
Max_miso Alias Pinb.0 : Set Max_miso

Config Portd = &B11110001                                   'PortD.7,6,5,4 a 1 = out, 1,2,3 = in (0-in, 1-out)
Piezo Alias Portd.4                                         'piezosirenka   (aktivni v log.1)
Zelena Alias Portd.6                                        'LED - zelena   (aktivni v log.1)
Cervena Alias Portd.5                                       'LED - cervena  (aktivni v log.1)
Podsviceni Alias Portd.7                                    'podsviceni LCD (aktivni v log.1)
Reset Piezo
Reset Zelena
Reset Cervena

Config Pind.1 = Input                                       'smrov vstup z Rotacniho koderu
Config Int0 = Falling                                       'Konfigurace peruen INT0, reaguj na sestupnou hranu (rot. koder)
Config Int1 = Falling                                       'Konfigurace peruen INT1, reaguj na sestupnou hranu (tlacitko)

Disable Interrupts                                          'zaka vechna peruen
Enable Int0                                                 'povol peruen INT0
Enable Int1                                                 'povol peruen INT1

On Timer1 Kontrola                                          'obsluha peruen od Timeru1
On Int0 Koder                                               'obsluha peruen od INT0
On Int1 Tlacitko                                            'obsluha peruen od INT1

Declare Sub Provoz()
Declare Sub Standby()
Declare Sub Nacteni_teploty()
Declare Sub Menu()
Declare Sub Kalibrace()

Deflcdchar 1 , 16 , 6 , 9 , 8 , 8 , 9 , 6 , 32              ' C
Deflcdchar 2 , 10 , 10 , 10 , 10 , 17 , 17 , 17 , 14        ' teplomer_dolni_prazdny
Deflcdchar 3 , 10 , 10 , 10 , 14 , 31 , 31 , 31 , 14        ' teplomer_dolni_pulka
Deflcdchar 4 , 14 , 14 , 14 , 14 , 31 , 31 , 31 , 14        ' teplomer_dolni_plny
Deflcdchar 5 , 4 , 10 , 10 , 10 , 10 , 10 , 10 , 10         ' teplomer_horni_prazdny
Deflcdchar 6 , 4 , 10 , 10 , 10 , 14 , 14 , 14 , 14         ' teplomer_horni_pulka
Deflcdchar 7 , 4 , 14 , 14 , 14 , 14 , 14 , 14 , 14         ' teplomer_horni_plny

Dim Teplota_vypocet As Single                               'promnn pro vpoet teploty
Dim Kalibr As Single
Dim Teplota As Word                                         'upravena vysledna teplota
Dim Teplota_max6675 As Word                                 'nactena teplota z MAX6675
Dim Tempset As Word , Dif As Integer                        'promnn pro regulan odchylku
Dim Smer As Bit , Tlac As Bit                               'pomocn promnn
Dim Prvnizahrati As Bit                                     'po zapnuti muze po dosazeni teploty pipnout piezo
Dim Pwm As Byte                                             'promnn pro zpis do PWM
Dim Odlozeno As Word                                        'doba do vypnuti vyhrivani pri delsi neccinosti
Dim Inmenu As Byte
Dim Teplodole As Byte
Dim Teplohore As Byte

'promnn pro uloen do eeprom
Dim Piezosave As Eram Byte At 1
Dim Odlozenisave As Eram Byte At 2
Dim Tempsave As Eram Word
Dim Kalibrsave As Eram Single

'po naprogramovani nastav maximalni hodnoty promenych
If Tempsave > 450 Then
   Tempsave = 450                                           'omezen maximln teploty po naprogramovani procesoru
   Piezosave = 1
   Odlozenisave = 1
   Kalibrsave = 0.95                                        'prednastavena kalibrace teploty
End If

Kalibr = Kalibrsave
Tempset = Tempsave
Odlozeno = 0
Prvnizahrati = 1                                            'prednastaveni pro zvukovou indikaci dosazeni teploty pri prvnim zahrati
'-----------------------------------------------------------------------------------------------------------------

If Pind.3 = 0 Then Gosub Kalibrace

'ihned po zapnuti zahrivej - zkraceni doby potrebne pro nahrati
Gosub Nacteni_teploty
Waitms 100
Gosub Nacteni_teploty
If Teplota < 80 Then
   Topeni = 255                                             'kdyz je termoclanek pripojen a neni zahraty (min nez 80C) tak top
   Set Cervena
End If

Set Podsviceni                                              'zapni podsviceni LCD

Cls                                                         'vyma displej
Locate 1 , 2 : Lcd "Pajeci stanice"                         'na prvn dek, druh sloupec
Locate 2 , 5 : Lcd "* PaJa *"                               'na druh dek, pt sloupec
Wait 1                                                      'pokej 1 sekundu

Cls
Locate 1 , 4 : Lcd "mikroPaJka"
Locate 2 , 7 : Lcd "2011  v1.0"
Wait 2

If Piezosave = 1 Then
   Set Piezo : Waitms 100 : Reset Piezo
End If

Cls
Locate 1 , 1 : Lcd Chr(5) ; "  SET "                        'znak z tabulky 5 (teplomr)
Locate 2 , 1 : Lcd Chr(2) ; "  HROT "
Locate 1 , 16 : Lcd Chr(5)
Locate 2 , 16 : Lcd Chr(2)

Enable Interrupts                                           'povol vechna peruen
Start Timer1                                                'spus timer1
Start Timer2                                                'spus timer2 - PWM

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
''                                                    Hlavn program                                                                 ''
''                                                                                                                                   ''
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Do
   Nop                                                      'opakuje nekonenou smyku a ek na peruen (od kodru, tlatka, asovae)
Loop

End

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''Obsluha peruen, podprogramy'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

'-------------------------------------------------------------------------------------------------------------------
'podprogram detekce aktivity modu stendby (peruen od timeru1 kadch asi 0.5s)

Kontrola:                                                   'nzev podprogramu

Timer1 = 42005                                              'znovunastaveni registru TCNT1 (pro preruseni kazdou 0,5s)

If Odlozenisave = 1 Then
 If Stojanek = 0 Then                                       'je-li odlozena rucka ve stojanku
   Incr Odlozeno                                            'pocitej cas odlozene rucky pajky
   If Odlozeno = 1200 Or Odlozeno = 1440 Or Odlozeno = 1680 Then       'rucka je odlozena 10minut, 12minut ,14 minut
      Stop Timer1
      Set Piezo : Set Podsviceni : Waitms 500               'tak piskej a blikej podsvicenim
      Reset Piezo : Reset Podsviceni : Waitms 250
      Set Piezo : Set Podsviceni : Waitms 500
      Reset Piezo
      If Tlac = 1 Then Reset Podsviceni                     'v modu stand-by znovu vypni podsviceni
      Start Timer1
   End If

   If Odlozeno > 1800 Then                                  'po case necinosti > 15min vypni Timer1, topeni, podsviceni, rozsvit zelenou LED
      Stop Timer1
      Disable Int0                                          'vypni extern peruen INT0
      Topeni = 0 : Reset Cervena : Set Zelena : Reset Podsviceni
      Cls
      Locate 1 , 1 : Lcd Chr(5) ; "Odstav. > 15min"         'a na LCD vypis Odsvateno vic jak 15minut, hrot vypnut
      Locate 2 , 1 : Lcd Chr(2) ; "  HROT vypnut"           'probuzeni teprve az po vyjmuti rucky ze stojanku s tisku tlacitka
      Goto Preskoc                                          'preskoc kontrolu Tlac a tim i podprogramy provoz a standby
   End If
 Else
   Odlozeno = 0
 End If
End If

If Tlac = 0 Then                                            'pokud je promnn 0 tak (stendby vypnuto)
   Gosub Provoz                                             'pokrauj do podprogramu provozu
Elseif Tlac = 1 Then                                        'pokud je promnn 1 (standby zapnuto) jdi do podrogramu standby
   Gosub Standby
End If                                                      'konec podmnky

Preskoc:
Return                                                      'ukonen peruen

'-------------------------------------------------------------------------------------------------------------------


'----------------------------------- Provozni rezim------------------------------------------------------------------
Sub Provoz()

Gosub Nacteni_teploty

Dif = Tempset - Teplota                                     'vpet regulan odchylky

If Prvnizahrati = 1 Then                                    'pri prvnim zahrati
   If Dif <= 1 Then                                         'a dosazeni nastavene teploty
         Prvnizahrati = 0                                   'vynuluj priznak prvniho zahrati
         If Piezosave = 1 Then                              'a v pripade povoleni pieza 2x zapiskej
            Set Piezo : Waitms 100 : Reset Piezo : Waitms 300
            Set Piezo : Waitms 350 : Reset Piezo
         End If
   End If
End If

Select Case Dif
   Case Is > 2 : Reset Zelena                               'zhasni  LED zelena
                  Set Cervena                               'rozsvit LED cervena
                  Teplodole = 4                             ' teplomer_dolni_plny
                  Teplohore = 5                             ' teplomer_horni_prazdny
   Case 0 To 2 : Set Zelena                                 ' zel + cer = oranzova LED
                  Set Cervena
                  Teplodole = 4                             ' teplomer_dolni_plny
                  Teplohore = 6                             ' teplomer_horni_pulka
   Case Is < 0 : Set Zelena
                  Reset Cervena
                  Teplodole = 4                             ' teplomer_dolni_plny
                  Teplohore = 7                             ' teplomer_horni_plny
End Select

Dif = Dif * 5                                               'strmost regulace PWM

Select Case Dif
   Case 0 To 150 : Pwm = Dif                                'pokud je strmost men jak 255 tak reguluj pomoc PWM  /  pevod na vhodn typ promnn (Byte)
   Case Is => 150 : Pwm = 255                               'pokud je strmost vt jak 150 tak trvale top
   Case Is < 0 : Pwm = 0                                    'pokud je regulan odchylka zporn (petopeno) vypni topen
End Select
Topeni = Pwm

   Locate 1 , 1 : Lcd Chr(teplohore) ; "  SET  " ; Tempset ; " " ; Chr(1) ; "  "
   Locate 2 , 1 : Lcd Chr(teplodole) ; "  HROT " ; Teplota ; " " ; Chr(1) ; "  "
   Locate 1 , 16 : Lcd Chr(teplohore)
   Locate 2 , 16 : Lcd Chr(teplodole)

   If Teplota > 700 Then                                    'kontrola pripojeni termoclanku (neboli prekroceni teploty pres 700C)
      Locate 2 , 9 : Lcd "Odpojen"
      If Piezosave = 1 Then                                 'v pripade poruchy termoclanku a povoleni pieza
         Set Piezo : Waitms 500 : Reset Piezo               'zapiskej 0,5s
      End If
   End If

End Sub                                                     'konec podprogramu
'-------------------------------------------------------------------------------------------------------------------


'----------------------------------- Stand-by -----------------------------------------------------------------------
'princip je stejn, teplota ale nen zobrazovna a je udrovna na 120C

Sub Standby()

Gosub Nacteni_teploty

Dif = Tempset - Teplota                                     'vpet regulan odchylky (Dif = 120 - teplota)
Dif = Dif * 5                                               'strmost regulace PWM

Select Case Dif
   Case 0 To 150 : Pwm = Dif                                'pokud je strmost men jak 255 tak reguluj pomoc PWM  /  pevod na vhodn typ promnn (Byte)
   Case Is => 150 : Pwm = 255                               'pokud je strmost vt jak 150 tak trvale top
   Case Is < 0 : Pwm = 0                                    'pokud je regulan odchylka zporn (petopeno) vypni topen
End Select
Topeni = Pwm

End Sub

'-------------------------------------------------------------------------------------------------------------------


'---------------------------------- mereni teploty z MAX 6675 ------------------------------------------------------
Sub Nacteni_teploty()

Topeni = 0                                                  'zajist aby v kadm ppad bylo toptko odpojeno
Waitms 5                                                    'pokej na ustlen proud (death time)

Reset Max_cs
Waitus 500
Shiftin Max_miso , Max_sck , Teplota_max6675 , 0 , 16       'nacti 16bitu z MAX6675 (info o teplote termoclanku)
Set Max_cs

Shift Teplota_max6675 , Right , 5                           'posun o 3 rady doprava (data o teplote zacinaji az na 3tim bitu), pak teplota/4 (meri na 0,25C)

Teplota_vypocet = Teplota_max6675 * Kalibr                  'prepocet hodnoty - dle skutecneho mereni na hrotu pres kapku cinu
Teplota = Teplota_vypocet                                   'prevod prepocitane hodnoty single na word

End Sub

'-------------------------------------------------------------------------------------------------------------------


'---------------------------------- Obsluha rotanho koderu -------------------------------------------------------
Koder:

Smer = Pind.1                                               'zjisti hodnotu smeroveho vstupu z koderu

If Smer = 1 Then                                            'pokad je smr oten vpravo tak
   Incr Tempset                                             'piti 1 k nastaven teplot
Elseif Smer = 0 Then                                        'pokad je smr oten vlevo tak
   Decr Tempset                                             'odeti 1 od nastaven teploty
End If

If Tempset < 80 Then Tempset = 80                           'omezen minimln teploty
If Tempset > 450 Then Tempset = 450                         'omezen maximln teploty
Tempsave = Tempset                                          'ulo do EEPROM

Locate 1 , 9 : Lcd Tempset ; " " ; Chr(1) ; " "             'zobraz nastavenou teplotu
Waitms 1

Return                                                      'nvrat z peruen

'-------------------------------------------------------------------------------------------------------------------


'---------------------------------- Obsluha stisku tlacitka --------------------------------------------------------
Tlacitko:

Stop Timer1
Waitms 10

'sleduj dlku stisku tlacitka, je-li delsi nez 600ms, vstup do menu
If Pind.3 = 0 Then
   Inmenu = 0
   Do
      Incr Inmenu
      Waitms 20
      If Inmenu >= 30 Then Gosub Menu
   Loop Until Pind.3 = 1
End If

If Inmenu <= 10 Then                                        'trval-li stisk pod 200ms prepni normal/standby
   Toggle Tlac                                              'tlatko bylo stisknuto neguj hodnotu pedchozho stavu promnn
End If

If Tlac = 0 Then                                            'pokud se pechz do aktvnho modu tak

   Set Podsviceni                                           'zapni podsviceni LCD
   Tempset = Tempsave                                       'nacti teplotu z pameti
   Enable Int0                                              'povol extern peruen INT0 (enkoder)

   Cls                                                      'vyma LCD
   Locate 1 , 1 : Lcd Chr(teplohore) ; "  SET  " ; Tempset ; " " ; Chr(1) ; " "
   Locate 2 , 1 : Lcd Chr(teplodole) ; "  HROT " ; Teplota ; " " ; Chr(1) ; " "
   Locate 1 , 16 : Lcd Chr(teplohore)
   Locate 2 , 16 : Lcd Chr(teplodole)

Elseif Tlac = 1 Then                                        'pokad se pechz do stendby tak

   Set Zelena
   Set Cervena                                              'zelen + erven = lut LED
   Reset Podsviceni                                         'vypni podsviceni LCD

   Disable Int0                                             'vypni extern peruen INT0
   Tempset = 120                                            'nastav klidovou teplotu na 120C

   Cls
   Locate 1 , 1 : Lcd Chr(5) ; "   Stand-by   " ; Chr(5)
   Locate 2 , 1 : Lcd Chr(3) ; "     120 " ; Chr(1) ; "    " ; Chr(3)

End If                                                      'konec podmnky


If Piezosave = 1 Then
   Set Piezo : Waitms 50 : Reset Piezo
End If

If Odlozeno > 1800 Then Odlozeno = 0                        ' pri neccinosti > 15min se stiskem tlac. tato doba vynuluje

Start Timer1

Disable Int1                                                'zaka extern peruen INT1 (tlacitko)
Bitwait Pind.3 , Set
Enable Int1                                                 'zaka extern peruen INT1 (tlacitko)

Return                                                      'nvrat z peruen
'-------------------------------------------------------------------------------------------------------------------


'-------------------------------------- menu -----------------------------------------------------------------------
Sub Menu()

Set Podsviceni

Stop Timer1
Disable Int1                                                'zaka extern peruen INT1 (tlacitko)
Disable Int0                                                'zaka extern peruen INT0 (enkoder)


   Cls
   Locate 1 , 1 : Lcd "Menu - nastaveni"
   Waitms 500

'---- rychla zmena teploty ------------
   Locate 1 , 1 : Lcd "Nast.tepl.+- 50" ; Chr(1)
   Locate 2 , 1 : Lcd "fast SET: " ; Tempset ; " " ; Chr(1)

   Do
      Locate 2 , 11 : Lcd Tempset ; " " ; Chr(1) ; " "
      If Pind.2 = 0 Then
         Smer = Pind.1                                      'zjisti hodnotu smeroveho vstupu z koderu
         Bitwait Pind.2 , Set

         If Smer = 1 Then
            Tempset = Tempset + 50
            If Tempset > 450 Then Tempset = 450
         Else
            Tempset = Tempset - 50
            If Tempset < 80 Then Tempset = 80
         End If
      End If
      Waitms 10
   Loop Until Pind.3 = 0

   Tempsave = Tempset
   Locate 2 , 14 : Lcd " OK"
   Waitms 500
'---- /rychla zmena teploty ---------

'---- piezo ------------
   Cls
   Locate 1 , 1 : Lcd "Piezo:"

   Do
      If Piezosave = 0 Then
         Locate 2 , 1 : Lcd "Off - vypnout "
      Else
         Locate 2 , 1 : Lcd "On  - zapnout"
      End If

      If Pind.2 = 0 Then
         Smer = Pind.1                                      'zjisti hodnotu smeroveho vstupu z koderu
         Bitwait Pind.2 , Set
         If Smer = 1 Then
            Piezosave = 0
         Else
            Piezosave = 1
         End If
      End If
   Loop Until Pind.3 = 0

   Locate 1 , 1 : Lcd "Piezo: save   "
   Waitms 500
'---- /piezo ---------

'---- odlozeni ------------
   Cls
   Locate 1 , 1 : Lcd "Odlozeni rucky"
   Do
      If Odlozenisave = 0 Then
         Locate 2 , 1 : Lcd "Off - Nehlidat "
      Else
         Locate 2 , 1 : Lcd "On  - Hlidat   "
      End If

      If Pind.2 = 0 Then
         Smer = Pind.1                                      'zjisti hodnotu smeroveho vstupu z koderu
         Bitwait Pind.2 , Set

         If Smer = 1 Then
            Odlozenisave = 0
         Else
            Odlozenisave = 1
         End If
      End If
   Loop Until Pind.3 = 0

   Locate 2 , 1 : Lcd "Odlozeni: save "
   Waitms 500
'---- /odlozeni ---------


Start Timer1
Enable Int0
Enable Int1

Inmenu = 20
Tlac = 0
Waitms 1

End Sub

'-------------------------------------------------------------------------------------------------------------------


'-------------------------------------- prvni menu nastaveni kalibrace teploty ----------------------------------------
Sub Kalibrace()

Set Podsviceni

Stop Timer1
Disable Int1                                                'zaka extern peruen INT1 (tlacitko)
Disable Int0                                                'zaka extern peruen INT0 (enkoder)

Kalibr = Kalibrsave

   Cls
   Locate 1 , 1 : Lcd "Kalibrace teplot"
   Locate 2 , 1 : Lcd "Tep.hrotu * " ; Kalibr

Bitwait Pind.3 , Set

   Do
      Locate 2 , 13 : Lcd Kalibr
      If Pind.2 = 0 Then
         Smer = Pind.1                                      'zjisti hodnotu smeroveho vstupu z koderu
         Bitwait Pind.2 , Set

         If Smer = 1 Then
            Kalibr = Kalibr + 0.01
            If Kalibr > 1.1 Then Kalibr = 1.1
         Else
            Kalibr = Kalibr - 0.01
            If Kalibr < 0.9 Then Kalibr = 0.9
         End If
      End If

      Waitms 5
   Loop Until Pind.3 = 0

   Kalibrsave = Kalibr
   Locate 2 , 13 : Lcd "Save"
   Wait 1

Start Timer1
Enable Int0
Enable Int1
Tlac = 0

End Sub

'-------------------------------------------------------------------------------------------------------------------


'******************************************* KONEC *****************************************************************